@@ -43,7 +43,7 @@
  * - Set operand size to 64 bits (W bit).
  * - Extend the register fields to access R8-R15 (R, X, B bits).
  */
-void emit_rex_prefix(code_buffer * buf, bool w, bool r, bool x, bool b) {
+INFIX_INTERNAL void emit_rex_prefix(code_buffer * buf, bool w, bool r, bool x, bool b) {
     uint8_t rex_byte = 0x40;
     if (w)
         rex_byte |= REX_W;
@@ -61,7 +61,7 @@ void emit_rex_prefix(code_buffer * buf, bool w, bool r, bool x, bool b) {
  * @details The ModR/M byte is a crucial part of many instructions, specifying the addressing mode.
  * It encodes register operands and memory operands.
  */
-void emit_modrm(code_buffer * buf, uint8_t mod, uint8_t reg_opcode, uint8_t rm) {
+INFIX_INTERNAL void emit_modrm(code_buffer * buf, uint8_t mod, uint8_t reg_opcode, uint8_t rm) {
     uint8_t modrm_byte = (mod << 6) | (reg_opcode << 3) | rm;
     emit_byte(buf, modrm_byte);
 }
@@ -72,7 +72,7 @@ void emit_modrm(code_buffer * buf, uint8_t mod, uint8_t reg_opcode, uint8_t rm)
  * @details Instruction Breakdown: MOV r64, imm64
  * Opcode format: REX.W + B8+rd imm64
  */
-void emit_mov_reg_imm64(code_buffer * buf, x64_gpr reg, uint64_t imm) {
+INFIX_INTERNAL void emit_mov_reg_imm64(code_buffer * buf, x64_gpr reg, uint64_t imm) {
     emit_rex_prefix(buf, 1, 0, 0, reg >= R8_REG);
     emit_byte(buf, 0xB8 + (reg % 8));
     emit_int64(buf, imm);
@@ -83,7 +83,7 @@ void emit_mov_reg_imm64(code_buffer * buf, x64_gpr reg, uint64_t imm) {
  * @details Instruction Breakdown: MOV r/m64, imm32 (sign-extended)
  * Opcode format: REX.W + C7 /0 id
  */
-void emit_mov_reg_imm32(code_buffer * buf, x64_gpr reg, int32_t imm) {
+INFIX_INTERNAL void emit_mov_reg_imm32(code_buffer * buf, x64_gpr reg, int32_t imm) {
     emit_rex_prefix(buf, 1, 0, 0, reg >= R8_REG);
     emit_byte(buf, 0xC7);
     emit_modrm(buf, 3, 0, reg % 8);  // mod=11 (register), reg=/0
@@ -96,7 +96,7 @@ void emit_mov_reg_imm32(code_buffer * buf, x64_gpr reg, int32_t imm) {
  * @details Instruction Breakdown: MOV r/m64, r64
  * Opcode format: REX.W + 89 /r
  */
-void emit_mov_reg_reg(code_buffer * buf, x64_gpr dest, x64_gpr src) {
+INFIX_INTERNAL void emit_mov_reg_reg(code_buffer * buf, x64_gpr dest, x64_gpr src) {
     uint8_t rex = REX_W;
     if (dest >= R8_REG)
         rex |= REX_B;
@@ -113,7 +113,7 @@ void emit_mov_reg_reg(code_buffer * buf, x64_gpr dest, x64_gpr src) {
  * @details Instruction Breakdown: MOV r64, r/m64
  * Opcode format: REX.W + 8B /r
  */
-void emit_mov_reg_mem(code_buffer * buf, x64_gpr dest, x64_gpr src_base, int32_t offset) {
+INFIX_INTERNAL void emit_mov_reg_mem(code_buffer * buf, x64_gpr dest, x64_gpr src_base, int32_t offset) {
     emit_rex_prefix(buf, 1, dest >= R8_REG, 0, src_base >= R8_REG);
     emit_byte(buf, 0x8B);
     uint8_t mod = (offset >= -128 && offset <= 127) ? 0x40 : 0x80;
@@ -132,7 +132,7 @@ void emit_mov_reg_mem(code_buffer * buf, x64_gpr dest, x64_gpr src_base, int32_t
  * @brief Emits `mov r32, r/m32` to load a 32-bit value (zero-extended to 64).
  * @details Opcode format: 8B /r (without REX.W)
  */
-void emit_mov_reg32_mem(code_buffer * buf, x64_gpr dest, x64_gpr src_base, int32_t offset) {
+INFIX_INTERNAL void emit_mov_reg32_mem(code_buffer * buf, x64_gpr dest, x64_gpr src_base, int32_t offset) {
     uint8_t rex = 0;
     if (dest >= R8_REG)
         rex |= REX_R;
@@ -157,7 +157,7 @@ void emit_mov_reg32_mem(code_buffer * buf, x64_gpr dest, x64_gpr src_base, int32
  * @brief Emits `movsxd r64, r/m32` to load a 32-bit value and sign-extend to 64.
  * @details Opcode format: REX.W + 63 /r
  */
-void emit_movsxd_reg_mem(code_buffer * buf, x64_gpr dest, x64_gpr src_base, int32_t offset) {
+INFIX_INTERNAL void emit_movsxd_reg_mem(code_buffer * buf, x64_gpr dest, x64_gpr src_base, int32_t offset) {
     emit_rex_prefix(buf, 1, dest >= R8_REG, 0, src_base >= R8_REG);
     emit_byte(buf, 0x63);
     uint8_t mod = (offset >= -128 && offset <= 127) ? 0x40 : 0x80;
@@ -176,7 +176,7 @@ void emit_movsxd_reg_mem(code_buffer * buf, x64_gpr dest, x64_gpr src_base, int3
  * @brief Emits `movsx r64, r/m8` to load a signed 8-bit value and sign-extend to 64.
  * @details Opcode format: REX.W + 0F BE /r
  */
-void emit_movsx_reg64_mem8(code_buffer * buf, x64_gpr dest, x64_gpr src_base, int32_t offset) {
+INFIX_INTERNAL void emit_movsx_reg64_mem8(code_buffer * buf, x64_gpr dest, x64_gpr src_base, int32_t offset) {
     emit_rex_prefix(buf, 1, dest >= R8_REG, 0, src_base >= R8_REG);
     EMIT_BYTES(buf, 0x0F, 0xBE);
     uint8_t mod = (offset >= -128 && offset <= 127) ? 0x40 : 0x80;
@@ -195,7 +195,7 @@ void emit_movsx_reg64_mem8(code_buffer * buf, x64_gpr dest, x64_gpr src_base, in
  * @brief Emits `movsx r64, r/m16` to load a signed 16-bit value and sign-extend to 64.
  * @details Opcode format: REX.W + 0F BF /r
  */
-void emit_movsx_reg64_mem16(code_buffer * buf, x64_gpr dest, x64_gpr src_base, int32_t offset) {
+INFIX_INTERNAL void emit_movsx_reg64_mem16(code_buffer * buf, x64_gpr dest, x64_gpr src_base, int32_t offset) {
     emit_rex_prefix(buf, 1, dest >= R8_REG, 0, src_base >= R8_REG);
     EMIT_BYTES(buf, 0x0F, 0xBF);
     uint8_t mod = (offset >= -128 && offset <= 127) ? 0x40 : 0x80;
@@ -214,7 +214,7 @@ void emit_movsx_reg64_mem16(code_buffer * buf, x64_gpr dest, x64_gpr src_base, i
  * @brief Emits `movzx r64, r/m8` to load an unsigned 8-bit value and zero-extend to 64.
  * @details Opcode format: REX.W + 0F B6 /r
  */
-void emit_movzx_reg64_mem8(code_buffer * buf, x64_gpr dest, x64_gpr src_base, int32_t offset) {
+INFIX_INTERNAL void emit_movzx_reg64_mem8(code_buffer * buf, x64_gpr dest, x64_gpr src_base, int32_t offset) {
     emit_rex_prefix(buf, 1, dest >= R8_REG, 0, src_base >= R8_REG);
     EMIT_BYTES(buf, 0x0F, 0xB6);
     uint8_t mod = (offset >= -128 && offset <= 127) ? 0x40 : 0x80;
@@ -233,7 +233,7 @@ void emit_movzx_reg64_mem8(code_buffer * buf, x64_gpr dest, x64_gpr src_base, in
  * @brief Emits `movzx r64, r/m16` to load an unsigned 16-bit value and zero-extend to 64.
  * @details Opcode format: REX.W + 0F B7 /r
  */
-void emit_movzx_reg64_mem16(code_buffer * buf, x64_gpr dest, x64_gpr src_base, int32_t offset) {
+INFIX_INTERNAL void emit_movzx_reg64_mem16(code_buffer * buf, x64_gpr dest, x64_gpr src_base, int32_t offset) {
     emit_rex_prefix(buf, 1, dest >= R8_REG, 0, src_base >= R8_REG);
     EMIT_BYTES(buf, 0x0F, 0xB7);
     uint8_t mod = (offset >= -128 && offset <= 127) ? 0x40 : 0x80;
@@ -253,7 +253,7 @@ void emit_movzx_reg64_mem16(code_buffer * buf, x64_gpr dest, x64_gpr src_base, i
  * @brief Emits `mov [base + offset], r64` to store a 64-bit GPR to memory.
  * @details Opcode format: REX.W + 89 /r
  */
-void emit_mov_mem_reg(code_buffer * buf, x64_gpr dest_base, int32_t offset, x64_gpr src) {
+INFIX_INTERNAL void emit_mov_mem_reg(code_buffer * buf, x64_gpr dest_base, int32_t offset, x64_gpr src) {
     emit_rex_prefix(buf, 1, src >= R8_REG, 0, dest_base >= R8_REG);
     emit_byte(buf, 0x89);
     uint8_t mod = (offset >= -128 && offset <= 127) ? 0x40 : 0x80;
@@ -272,7 +272,7 @@ void emit_mov_mem_reg(code_buffer * buf, x64_gpr dest_base, int32_t offset, x64_
  * @brief Emits `mov [base + offset], r32` to store a 32-bit GPR to memory.
  * @details Opcode format: 89 /r (without REX.W)
  */
-void emit_mov_mem_reg32(code_buffer * buf, x64_gpr dest_base, int32_t offset, x64_gpr src) {
+INFIX_INTERNAL void emit_mov_mem_reg32(code_buffer * buf, x64_gpr dest_base, int32_t offset, x64_gpr src) {
     uint8_t rex = 0;
     if (src >= R8_REG)
         rex |= REX_R;
@@ -297,7 +297,7 @@ void emit_mov_mem_reg32(code_buffer * buf, x64_gpr dest_base, int32_t offset, x6
  * @brief Emits `mov [base + offset], r16` to store a 16-bit GPR to memory.
  * @details Opcode format: 66 + 89 /r (66 is the operand-size override prefix).
  */
-void emit_mov_mem_reg16(code_buffer * buf, x64_gpr dest_base, int32_t offset, x64_gpr src) {
+INFIX_INTERNAL void emit_mov_mem_reg16(code_buffer * buf, x64_gpr dest_base, int32_t offset, x64_gpr src) {
     emit_byte(buf, 0x66);
     uint8_t rex = 0;
     if (src >= R8_REG)
@@ -323,7 +323,7 @@ void emit_mov_mem_reg16(code_buffer * buf, x64_gpr dest_base, int32_t offset, x6
  * @brief Emits `mov [base + offset], r8` to store an 8-bit GPR to memory.
  * @details Opcode format: 88 /r
  */
-void emit_mov_mem_reg8(code_buffer * buf, x64_gpr dest_base, int32_t offset, x64_gpr src) {
+INFIX_INTERNAL void emit_mov_mem_reg8(code_buffer * buf, x64_gpr dest_base, int32_t offset, x64_gpr src) {
     uint8_t rex = 0;
     if (src >= R8_REG || dest_base >= R8_REG || src >= RSP_REG)
         rex = 0x40;
@@ -351,7 +351,7 @@ void emit_mov_mem_reg8(code_buffer * buf, x64_gpr dest_base, int32_t offset, x64
  * @brief Emits `movss xmm, [base + offset]` to load a 32-bit float from memory.
  * @details Opcode format: F3 0F 10 /r
  */
-void emit_movss_xmm_mem(code_buffer * buf, x64_xmm dest, x64_gpr src_base, int32_t offset) {
+INFIX_INTERNAL void emit_movss_xmm_mem(code_buffer * buf, x64_xmm dest, x64_gpr src_base, int32_t offset) {
     emit_byte(buf, 0xF3);
     uint8_t rex = 0;
     if (dest >= XMM8_REG)
@@ -377,7 +377,7 @@ void emit_movss_xmm_mem(code_buffer * buf, x64_xmm dest, x64_gpr src_base, int32
  * @brief Emits `movss [base + offset], xmm` to store a 32-bit float to memory.
  * @details Opcode format: F3 0F 11 /r
  */
-void emit_movss_mem_xmm(code_buffer * buf, x64_gpr dest_base, int32_t offset, x64_xmm src) {
+INFIX_INTERNAL void emit_movss_mem_xmm(code_buffer * buf, x64_gpr dest_base, int32_t offset, x64_xmm src) {
     emit_byte(buf, 0xF3);
     uint8_t rex = 0;
     if (src >= XMM8_REG)
@@ -403,7 +403,7 @@ void emit_movss_mem_xmm(code_buffer * buf, x64_gpr dest_base, int32_t offset, x6
  * @brief Emits `movsd xmm, [base + offset]` to load a 64-bit double from memory.
  * @details Opcode format: F2 0F 10 /r
  */
-void emit_movsd_xmm_mem(code_buffer * buf, x64_xmm dest, x64_gpr src_base, int32_t offset) {
+INFIX_INTERNAL void emit_movsd_xmm_mem(code_buffer * buf, x64_xmm dest, x64_gpr src_base, int32_t offset) {
     emit_byte(buf, 0xF2);
     uint8_t rex = 0;
     if (dest >= XMM8_REG)
@@ -429,7 +429,7 @@ void emit_movsd_xmm_mem(code_buffer * buf, x64_xmm dest, x64_gpr src_base, int32
  * @brief Emits `movsd [base + offset], xmm` to store a 64-bit double to memory.
  * @details Opcode format: F2 0F 11 /r
  */
-void emit_movsd_mem_xmm(code_buffer * buf, x64_gpr dest_base, int32_t offset, x64_xmm src) {
+INFIX_INTERNAL void emit_movsd_mem_xmm(code_buffer * buf, x64_gpr dest_base, int32_t offset, x64_xmm src) {
     emit_byte(buf, 0xF2);
     uint8_t rex = 0;
     if (src >= XMM8_REG)
@@ -455,7 +455,7 @@ void emit_movsd_mem_xmm(code_buffer * buf, x64_gpr dest_base, int32_t offset, x6
  * @brief Emits `movups xmm, [base + offset]` to load a 128-bit unaligned value from memory.
  * @details Opcode format: 0F 10 /r
  */
-void emit_movups_xmm_mem(code_buffer * buf, x64_xmm dest, x64_gpr src_base, int32_t offset) {
+INFIX_INTERNAL void emit_movups_xmm_mem(code_buffer * buf, x64_xmm dest, x64_gpr src_base, int32_t offset) {
     uint8_t rex = 0;
     if (dest >= XMM8_REG)
         rex |= REX_R;
@@ -480,7 +480,7 @@ void emit_movups_xmm_mem(code_buffer * buf, x64_xmm dest, x64_gpr src_base, int3
  * @brief Emits `movups [base + offset], xmm` to store a 128-bit unaligned value to memory.
  * @details Opcode format: 0F 11 /r
  */
-void emit_movups_mem_xmm(code_buffer * buf, x64_gpr dest_base, int32_t offset, x64_xmm src) {
+INFIX_INTERNAL void emit_movups_mem_xmm(code_buffer * buf, x64_gpr dest_base, int32_t offset, x64_xmm src) {
     uint8_t rex = 0;
     if (src >= XMM8_REG)
         rex |= REX_R;
@@ -505,7 +505,7 @@ void emit_movups_mem_xmm(code_buffer * buf, x64_gpr dest_base, int32_t offset, x
  * @brief Emits a VEX prefix for an AVX instruction.
  * @details This helper centralizes the logic of choosing between 2-byte (C5) and 3-byte (C4) VEX encodings.
  */
-static void emit_vex_prefix(
+INFIX_INTERNAL void emit_vex_prefix(
     code_buffer * buf, bool r, bool x, bool b, uint8_t m, bool w, uint8_t v, bool l, uint8_t p) {
     // VEX encoding inverts R, X, B bits.
     // The 2-byte VEX prefix cannot encode the L bit for 256-bit operations.
@@ -529,20 +529,20 @@ static void emit_vex_prefix(
  * @internal
  * @brief Emits a 4-byte EVEX prefix for an AVX-512 instruction, following the Intel SDM.
  */
-static void emit_evex_prefix(code_buffer * buf,
-                             uint8_t map,  // 1 for 0F, 2 for 0F38, 3 for 0F3A
-                             uint8_t pp,   // 00=none, 01=66, 10=F3, 11=F2
-                             bool W,
-                             bool R,
-                             bool X,
-                             bool B,
-                             bool R_prime,  // Register bits
-                             uint8_t vvvv,  // Source register (inverted)
-                             bool L,
-                             bool L_prime,
-                             bool z,
-                             bool b,
-                             uint8_t aaa)  // Masking/control bits
+INFIX_INTERNAL void emit_evex_prefix(code_buffer * buf,
+                                     uint8_t map,  // 1 for 0F, 2 for 0F38, 3 for 0F3A
+                                     uint8_t pp,   // 00=none, 01=66, 10=F3, 11=F2
+                                     bool W,
+                                     bool R,
+                                     bool X,
+                                     bool B,
+                                     bool R_prime,  // Register bits
+                                     uint8_t vvvv,  // Source register (inverted)
+                                     bool L,
+                                     bool L_prime,
+                                     bool z,
+                                     bool b,
+                                     uint8_t aaa)  // Masking/control bits
 {
     emit_byte(buf, 0x62);
     // Byte 2: P0 - R, X, B, R' bits are inverted. 0 means 1, 1 means 0.
@@ -576,7 +576,7 @@ static void emit_evex_prefix(code_buffer * buf,
  * @brief Emits `vmovupd ymm, [base + offset]` to load a 256-bit unaligned value (AVX).
  * @details Instruction format: VEX.256.66.0F.WIG 10 /r
  */
-void emit_vmovupd_ymm_mem(code_buffer * buf, x64_xmm dest, x64_gpr src_base, int32_t offset) {
+INFIX_INTERNAL void emit_vmovupd_ymm_mem(code_buffer * buf, x64_xmm dest, x64_gpr src_base, int32_t offset) {
     // VEX prefix fields for vmovupd ymm, m256:
     // L=1 (256-bit), p=1 (from 66 prefix), m-mmmm=01 (from 0F map).
     // The vvvv field is not used for a memory source and should be 0.
@@ -598,7 +598,7 @@ void emit_vmovupd_ymm_mem(code_buffer * buf, x64_xmm dest, x64_gpr src_base, int
  * @brief Emits `vmovupd [base + offset], ymm` to store a 256-bit unaligned value (AVX).
  * @details Instruction format: VEX.256.66.0F.WIG 11 /r
  */
-void emit_vmovupd_mem_ymm(code_buffer * buf, x64_gpr dest_base, int32_t offset, x64_xmm src) {
+INFIX_INTERNAL void emit_vmovupd_mem_ymm(code_buffer * buf, x64_gpr dest_base, int32_t offset, x64_xmm src) {
     // For a store, the VEX.vvvv field is not used and should be 0.
     emit_vex_prefix(buf, src >= XMM8_REG, 0, dest_base >= R8_REG, 1, false, 0, true, 1);
     emit_byte(buf, 0x11);  // Opcode for MOVUPD (store)
@@ -618,7 +618,7 @@ void emit_vmovupd_mem_ymm(code_buffer * buf, x64_gpr dest_base, int32_t offset,
  * @brief Emits `vmovupd zmm, [base + offset]` to load a 512-bit unaligned value (AVX-512).
  * @details Instruction format: EVEX.512.66.0F.W0 10 /r
  */
-void emit_vmovupd_zmm_mem(code_buffer * buf, x64_xmm dest, x64_gpr src_base, int32_t offset) {
+INFIX_INTERNAL void emit_vmovupd_zmm_mem(code_buffer * buf, x64_xmm dest, x64_gpr src_base, int32_t offset) {
     // For vmovupd zmm, m512:
     // vvvv field is unused and must be 0.
     emit_evex_prefix(buf,
@@ -654,7 +654,7 @@ void emit_vmovupd_zmm_mem(code_buffer * buf, x64_xmm dest, x64_gpr src_base, int
  * @brief Emits `vmovupd [base + offset], zmm` to store a 512-bit unaligned value (AVX-512).
  * @details Instruction format: EVEX.512.66.0F.W0 11 /r
  */
-void emit_vmovupd_mem_zmm(code_buffer * buf, x64_gpr dest_base, int32_t offset, x64_xmm src) {
+INFIX_INTERNAL void emit_vmovupd_mem_zmm(code_buffer * buf, x64_gpr dest_base, int32_t offset, x64_xmm src) {
     // For a store, the source register is encoded in EVEX.reg_field (via ModRM)
     // and the vvvv field is repurposed. Per Intel SDM, for a memory destination,
     // V' must be 1. We encode this by passing a value with the 5th bit set (16)
@@ -690,7 +690,7 @@ void emit_vmovupd_mem_zmm(code_buffer * buf, x64_gpr dest_base, int32_t offset,
  * @brief Emits `cvtsd2ss xmm1, xmm2/m64` to convert a double to a float.
  * @details Opcode format: F2 0F 5A /r
  */
-void emit_cvtsd2ss_xmm_xmm(code_buffer * buf, x64_xmm dest, x64_xmm src) {
+INFIX_INTERNAL void emit_cvtsd2ss_xmm_xmm(code_buffer * buf, x64_xmm dest, x64_xmm src) {
     emit_byte(buf, 0xF2);
     uint8_t rex = 0;
     if (dest >= XMM8_REG)
@@ -708,7 +708,7 @@ void emit_cvtsd2ss_xmm_xmm(code_buffer * buf, x64_xmm dest, x64_xmm src) {
  * @brief Emits `cvtsd2ss xmm, [base + offset]` to load a double, convert to float, and store in xmm.
  * @details Opcode format: F2 0F 5A /r
  */
-void emit_cvtsd2ss_xmm_mem(code_buffer * buf, x64_xmm dest, x64_gpr src_base, int32_t offset) {
+INFIX_INTERNAL void emit_cvtsd2ss_xmm_mem(code_buffer * buf, x64_xmm dest, x64_gpr src_base, int32_t offset) {
     emit_byte(buf, 0xF2);  // F2 prefix for SD (scalar double)
     uint8_t rex = 0;
     if (dest >= XMM8_REG)
@@ -740,7 +740,7 @@ void emit_cvtsd2ss_xmm_mem(code_buffer * buf, x64_xmm dest, x64_gpr src_base, in
  * @brief Emits `movaps xmm1, xmm2/m128` to move 128 bits between XMM registers.
  * @details Opcode format: 0F 28 /r
  */
-void emit_movaps_xmm_xmm(code_buffer * buf, x64_xmm dest, x64_xmm src) {
+INFIX_INTERNAL void emit_movaps_xmm_xmm(code_buffer * buf, x64_xmm dest, x64_xmm src) {
     uint8_t rex = 0;
     if (dest >= XMM8_REG)
         rex |= REX_R;
@@ -758,7 +758,7 @@ void emit_movaps_xmm_xmm(code_buffer * buf, x64_xmm dest, x64_xmm src) {
  * @brief Emits `movq xmm, r64` to move 64 bits from a GPR to an XMM register.
  * @details Opcode format: 66 + REX.W + 0F 6E /r
  */
-void emit_movq_xmm_gpr(code_buffer * buf, x64_xmm dest, x64_gpr src) {
+INFIX_INTERNAL void emit_movq_xmm_gpr(code_buffer * buf, x64_xmm dest, x64_gpr src) {
     emit_byte(buf, 0x66);
     emit_rex_prefix(buf, 1, dest >= XMM8_REG, 0, src >= R8_REG);
     EMIT_BYTES(buf, 0x0F, 0x6E);
@@ -769,7 +769,7 @@ void emit_movq_xmm_gpr(code_buffer * buf, x64_xmm dest, x64_gpr src) {
  * @brief Emits `movq r64, xmm` to move 64 bits from an XMM to a GPR.
  * @details Opcode format: 66 + REX.W + 0F 7E /r
  */
-void emit_movq_gpr_xmm(code_buffer * buf, x64_gpr dest, x64_xmm src) {
+INFIX_INTERNAL void emit_movq_gpr_xmm(code_buffer * buf, x64_gpr dest, x64_xmm src) {
     emit_byte(buf, 0x66);
     emit_rex_prefix(buf, 1, src >= XMM8_REG, 0, dest >= R8_REG);
     EMIT_BYTES(buf, 0x0F, 0x7E);
@@ -781,7 +781,7 @@ void emit_movq_gpr_xmm(code_buffer * buf, x64_gpr dest, x64_xmm src) {
  * @brief Emits `fldt [base + offset]` to load an 80-bit `long double` onto the FPU stack.
  * @details Opcode format: DB /5
  */
-void emit_fldt_mem(code_buffer * buf, x64_gpr base, int32_t offset) {
+INFIX_INTERNAL void emit_fldt_mem(code_buffer * buf, x64_gpr base, int32_t offset) {
     uint8_t rex = 0;
     if (base >= R8_REG)
         rex |= REX_B;
@@ -804,7 +804,7 @@ void emit_fldt_mem(code_buffer * buf, x64_gpr base, int32_t offset) {
  * @brief Emits `fstpt [base + offset]` to store and pop an 80-bit `long double`.
  * @details Opcode format: DB /7
  */
-void emit_fstpt_mem(code_buffer * buf, x64_gpr base, int32_t offset) {
+INFIX_INTERNAL void emit_fstpt_mem(code_buffer * buf, x64_gpr base, int32_t offset) {
     uint8_t rex = 0;
     if (base >= R8_REG)
         rex |= REX_B;
@@ -828,7 +828,7 @@ void emit_fstpt_mem(code_buffer * buf, x64_gpr base, int32_t offset) {
  * @brief Emits `lea r64, [base + offset]` to load an effective address.
  * @details Opcode format: REX.W + 8D /r
  */
-void emit_lea_reg_mem(code_buffer * buf, x64_gpr dest, x64_gpr src_base, int32_t offset) {
+INFIX_INTERNAL void emit_lea_reg_mem(code_buffer * buf, x64_gpr dest, x64_gpr src_base, int32_t offset) {
     emit_rex_prefix(buf, 1, dest >= R8_REG, 0, src_base >= R8_REG);
     emit_byte(buf, 0x8D);
     uint8_t mod = (offset >= -128 && offset <= 127) ? 0x40 : 0x80;
@@ -847,7 +847,7 @@ void emit_lea_reg_mem(code_buffer * buf, x64_gpr dest, x64_gpr src_base, int32_t
  * @brief Emits `add r64, imm32` to add a 32-bit immediate to a GPR.
  * @details Opcode format: REX.W + 81 /0 id
  */
-void emit_add_reg_imm32(code_buffer * buf, x64_gpr reg, int32_t imm) {
+INFIX_INTERNAL void emit_add_reg_imm32(code_buffer * buf, x64_gpr reg, int32_t imm) {
     emit_rex_prefix(buf, 1, 0, 0, reg >= R8_REG);
     emit_byte(buf, 0x81);
     emit_modrm(buf, 3, 0, reg % 8);  // mod=11, reg=/0 for ADD
@@ -858,7 +858,7 @@ void emit_add_reg_imm32(code_buffer * buf, x64_gpr reg, int32_t imm) {
  * @brief Emits `sub r64, imm32` to subtract a 32-bit immediate from a GPR.
  * @details Opcode format: REX.W + 81 /5 id
  */
-void emit_sub_reg_imm32(code_buffer * buf, x64_gpr reg, int32_t imm) {
+INFIX_INTERNAL void emit_sub_reg_imm32(code_buffer * buf, x64_gpr reg, int32_t imm) {
     emit_rex_prefix(buf, 1, 0, 0, reg >= R8_REG);
     emit_byte(buf, 0x81);
     emit_modrm(buf, 3, 5, reg % 8);  // mod=11, reg=/5 for SUB
@@ -869,7 +869,7 @@ void emit_sub_reg_imm32(code_buffer * buf, x64_gpr reg, int32_t imm) {
  * @brief Emits `add r/m64, imm8` (sign-extended) to a GPR.
  * @details Opcode format: REX.W + 83 /0 ib
  */
-void emit_add_reg_imm8(code_buffer * buf, x64_gpr reg, int8_t imm) {
+INFIX_INTERNAL void emit_add_reg_imm8(code_buffer * buf, x64_gpr reg, int8_t imm) {
     emit_rex_prefix(buf, 1, 0, 0, (reg >= R8_REG));
     emit_byte(buf, 0x83);
     emit_modrm(buf, 3, 0, (reg & 0x7));
@@ -880,7 +880,7 @@ void emit_add_reg_imm8(code_buffer * buf, x64_gpr reg, int8_t imm) {
  * @brief Emits `dec r/m64` to decrement a 64-bit register by 1.
  * @details Opcode format: REX.W + FF /1
  */
-void emit_dec_reg(code_buffer * buf, x64_gpr reg) {
+INFIX_INTERNAL void emit_dec_reg(code_buffer * buf, x64_gpr reg) {
     emit_rex_prefix(buf, 1, 0, 0, reg >= R8_REG);
     emit_byte(buf, 0xFF);
     emit_modrm(buf, 3, 1, reg % 8);
@@ -891,7 +891,7 @@ void emit_dec_reg(code_buffer * buf, x64_gpr reg) {
  * @brief Emits `push r64` to push a GPR onto the stack.
  * @details Opcode format: [REX.B] 50+rd
  */
-void emit_push_reg(code_buffer * buf, x64_gpr reg) {
+INFIX_INTERNAL void emit_push_reg(code_buffer * buf, x64_gpr reg) {
     uint8_t rex = 0;
     if (reg >= R8_REG)
         rex |= REX_B;
@@ -904,7 +904,7 @@ void emit_push_reg(code_buffer * buf, x64_gpr reg) {
  * @brief Emits `pop r64` to pop a 64-bit value from the stack into a register.
  * @details Opcode format: [REX.B] 58+rd
  */
-void emit_pop_reg(code_buffer * buf, x64_gpr reg) {
+INFIX_INTERNAL void emit_pop_reg(code_buffer * buf, x64_gpr reg) {
     uint8_t rex = 0;
     if (reg >= R8_REG)
         rex |= REX_B;
@@ -917,7 +917,7 @@ void emit_pop_reg(code_buffer * buf, x64_gpr reg) {
  * @brief Emits `call r64` to call a function pointer stored in a register.
  * @details Opcode format: [REX.W] [REX.B] FF /2
  */
-void emit_call_reg(code_buffer * buf, x64_gpr reg) {
+INFIX_INTERNAL void emit_call_reg(code_buffer * buf, x64_gpr reg) {
     uint8_t rex = REX_W;
     if (reg >= R8_REG)
         rex |= REX_B;
@@ -933,13 +933,13 @@ void emit_call_reg(code_buffer * buf, x64_gpr reg) {
  * @brief Emits `ret` to return from a function.
  * @details Opcode: C3
  */
-void emit_ret(code_buffer * buf) { emit_byte(buf, 0xC3); }
+INFIX_INTERNAL void emit_ret(code_buffer * buf) { emit_byte(buf, 0xC3); }
 /**
  * @internal
  * @brief Emits `test r64, r64` to test if a register is zero.
  * @details Opcode format: REX.W + 85 /r
  */
-void emit_test_reg_reg(code_buffer * buf, x64_gpr reg1, x64_gpr reg2) {
+INFIX_INTERNAL void emit_test_reg_reg(code_buffer * buf, x64_gpr reg1, x64_gpr reg2) {
     emit_rex_prefix(buf, 1, reg2 >= R8_REG, 0, reg1 >= R8_REG);
     emit_byte(buf, 0x85);
     emit_modrm(buf, 3, reg2 % 8, reg1 % 8);
@@ -949,14 +949,14 @@ void emit_test_reg_reg(code_buffer * buf, x64_gpr reg1, x64_gpr reg2) {
  * @brief Emits `jnz rel8` for a short conditional jump if not zero.
  * @details Opcode format: 75 rel8
  */
-void emit_jnz_short(code_buffer * buf, int8_t offset) { EMIT_BYTES(buf, 0x75, (uint8_t)offset); }
+INFIX_INTERNAL void emit_jnz_short(code_buffer * buf, int8_t offset) { EMIT_BYTES(buf, 0x75, (uint8_t)offset); }
 /**
  * @internal
  * @brief Emits a `jmp r64` instruction.
  * @details This instruction performs an indirect jump to the address contained in the
  * specified 64-bit register. Opcode format: [REX.B] FF /4
  */
-void emit_jmp_reg(code_buffer * buf, x64_gpr reg) {
+INFIX_INTERNAL void emit_jmp_reg(code_buffer * buf, x64_gpr reg) {
     uint8_t rex = 0;
     if (reg >= R8_REG)
         rex = 0x40 | REX_B;
@@ -970,17 +970,17 @@ void emit_jmp_reg(code_buffer * buf, x64_gpr reg) {
  * @brief Emits `ud2`, an undefined instruction that causes an invalid opcode exception.
  * @details Opcode format: 0F 0B
  */
-void emit_ud2(code_buffer * buf) { EMIT_BYTES(buf, 0x0F, 0x0B); }
+INFIX_INTERNAL void emit_ud2(code_buffer * buf) { EMIT_BYTES(buf, 0x0F, 0x0B); }
 /**
  * @internal
  * @brief Emits the two-byte `syscall` instruction.
  * @details Opcode: 0F 05
  */
-void emit_syscall(code_buffer * buf) { EMIT_BYTES(buf, 0x0F, 0x05); }
+INFIX_INTERNAL void emit_syscall(code_buffer * buf) { EMIT_BYTES(buf, 0x0F, 0x05); }
 /**
  * @internal
  * @brief Emits the `leave` instruction to tear down a stack frame.
  * @details This is equivalent to `mov rsp, rbp` followed by `pop rbp`.
  *          Opcode: C9
  */
-void emit_leave(code_buffer * buf) { emit_byte(buf, 0xC9); }
+INFIX_INTERNAL void emit_leave(code_buffer * buf) { emit_byte(buf, 0xC9); }
